@@ -1,16 +1,17 @@
+"""Pulse representation of a quantum circuit."""
+from packaging.version import parse as parse_version
 import numpy as np
 from scipy.interpolate import CubicSpline
 
-from qutip.qobjevo import QobjEvo
-from qutip.qobj import Qobj
+import qutip
+from qutip import QobjEvo, Qobj, identity
 from .operations import expand_operator
-from qutip.operators import identity
 
 
 __all__ = ["Pulse", "Drift"]
 
 
-class _EvoElement():
+class _EvoElement:
     """
     The class object saving the information of one evolution element.
     Each dynamic element is characterized by four variables:
@@ -19,6 +20,7 @@ class _EvoElement():
     For documentation and use instruction of the attributes, please
     refer to :class:`.Pulse`.
     """
+
     def __init__(self, qobj, targets, tlist=None, coeff=None):
         self.qobj = qobj
         self.targets = targets
@@ -46,7 +48,7 @@ class _EvoElement():
         if isinstance(dims, (int, np.integer)):
             dims = [2] * dims
         if self.qobj is None:
-            qobj = identity(dims[0]) * 0.
+            qobj = identity(dims[0]) * 0.0
             targets = 0
         else:
             qobj = self.qobj
@@ -59,28 +61,38 @@ class _EvoElement():
         """
         mat = self.get_qobj(dims)
         if self.tlist is None and self.coeff is None:
-            qu = QobjEvo(mat) * 0.
+            qu = QobjEvo(mat) * 0.0
         elif isinstance(self.coeff, bool):
             if self.coeff:
                 if self.tlist is None:
                     qu = QobjEvo(mat, tlist=self.tlist)
                 else:
-                    qu = QobjEvo([mat, np.ones(len(self.tlist))],
-                                 tlist=self.tlist)
+                    qu = QobjEvo(
+                        [mat, np.ones(len(self.tlist))], tlist=self.tlist
+                    )
             else:
-                qu = QobjEvo(mat * 0., tlist=self.tlist)
+                qu = QobjEvo(mat * 0.0, tlist=self.tlist)
         else:
-            if spline_kind == "step_func":
-                args = {"_step_func_coeff": True}
+            if spline_kind == "cubic":
+                qu = QobjEvo(
+                    [mat, self.coeff],
+                    tlist=self.tlist,
+                )
+            elif spline_kind == "step_func":
                 if len(self.coeff) == len(self.tlist) - 1:
-                    self.coeff = np.concatenate([self.coeff, [0.]])
-            elif spline_kind == "cubic":
-                args = {"_step_func_coeff": False}
+                    self.coeff = np.concatenate([self.coeff, [0.0]])
+                if parse_version(qutip.__version__) >= parse_version("5.dev"):
+                    qu = QobjEvo([mat, self.coeff], tlist=self.tlist, order=0)
+                else:
+                    qu = QobjEvo(
+                        [mat, self.coeff],
+                        tlist=self.tlist,
+                        args={"_step_func_coeff": True},
+                    )
             else:
                 # The spline will follow other pulses or
                 # use the default value of QobjEvo
-                args = {}
-            qu = QobjEvo([mat, self.coeff], tlist=self.tlist, args=args)
+                raise ValueError("The pulse has an unknown spline type.")
         return qu
 
     def get_qobjevo(self, spline_kind, dims):
@@ -119,31 +131,35 @@ class _EvoElement():
             return self._get_qobjevo_helper(spline_kind, dims=dims)
         except Exception as err:
             print(
-                "The Evolution element went wrong was\n {}".format(str(self)))
-            raise(err)
+                "The Evolution element went wrong was\n {}".format(str(self))
+            )
+            raise (err)
 
     def __str__(self):
-        return str({"qobj": self.qobj,
-                    "targets": self.targets,
-                    "tlist": self.tlist,
-                    "coeff": self.coeff
-                    })
+        return str(
+            {
+                "qobj": self.qobj,
+                "targets": self.targets,
+                "tlist": self.tlist,
+                "coeff": self.coeff,
+            }
+        )
 
 
-class Pulse():
+class Pulse:
     """
     Representation of a control pulse and the pulse dependent noise.
     The pulse is characterized by the ideal control pulse, the coherent
     noise and the lindblad noise. The later two are lists of
     noisy evolution dynamics.
     Each dynamic element is characterized by four variables:
-    ``1`qobj``, ``targets``, ``tlist`` and ``coeff``.
+    ``qobj``, ``targets``, ``tlist`` and ``coeff``.
 
     See examples for different construction behavior.
 
     Parameters
     ----------
-    qobj : :class:'qutip.Qobj'
+    qobj : :class:`qutip.Qobj`
         The Hamiltonian of the ideal pulse.
     targets: list
         target qubits of the ideal pulse
@@ -220,8 +236,16 @@ class Pulse():
     >>> Pulse(None, None) # doctest: +SKIP
 
     """
-    def __init__(self, qobj, targets, tlist=None, coeff=None,
-                 spline_kind=None, label=""):
+
+    def __init__(
+        self,
+        qobj,
+        targets,
+        tlist=None,
+        coeff=None,
+        spline_kind="step_func",
+        label="",
+    ):
         self.spline_kind = spline_kind
         self.ideal_pulse = _EvoElement(qobj, targets, tlist, coeff)
         self.coherent_noise = []
@@ -278,7 +302,7 @@ class Pulse():
 
         Parameters
         ----------
-        qobj: :class:'qutip.Qobj'
+        qobj: :class:`qutip.Qobj`
             The Hamiltonian of the pulse.
         targets: list
             target qubits of the pulse
@@ -309,7 +333,7 @@ class Pulse():
 
         Parameters
         ----------
-        qobj: :class:'qutip.Qobj'
+        qobj: :class:`qutip.Qobj`
             The collapse operator of the lindblad noise.
         targets: list
             target qubits of the collapse operator
@@ -388,11 +412,15 @@ class Pulse():
             A list of (time-dependent) lindbald operators.
         """
         ideal_qu = self.get_ideal_qobjevo(dims)
-        noise_qu_list = [noise.get_qobjevo(self.spline_kind, dims)
-                         for noise in self.coherent_noise]
+        noise_qu_list = [
+            noise.get_qobjevo(self.spline_kind, dims)
+            for noise in self.coherent_noise
+        ]
         qu = _merge_qobjevo([ideal_qu] + noise_qu_list)
-        c_ops = [noise.get_qobjevo(self.spline_kind, dims)
-                 for noise in self.lindblad_noise]
+        c_ops = [
+            noise.get_qobjevo(self.spline_kind, dims)
+            for noise in self.lindblad_noise
+        ]
         full_tlist = self.get_full_tlist()
         qu = _merge_qobjevo([qu], full_tlist)
         for i, c_op in enumerate(c_ops):
@@ -423,7 +451,8 @@ class Pulse():
             return None
         full_tlist = np.unique(np.sort(np.hstack(all_tlists)))
         full_tlist = np.concatenate(
-            (full_tlist[:1], full_tlist[1:][np.diff(full_tlist) > tol]))
+            (full_tlist[:1], full_tlist[1:][np.diff(full_tlist) > tol])
+        )
         return full_tlist
 
     def print_info(self):
@@ -431,13 +460,18 @@ class Pulse():
         Print the information of the pulse, including the ideal dynamics,
         the coherent noise and the lindblad noise.
         """
-        print("-----------------------------------"
-              "-----------------------------------")
+        print(
+            "-----------------------------------"
+            "-----------------------------------"
+        )
         if self.label is not None:
             print("Pulse label:", self.label)
-        print("The pulse contains: {} coherent noise elements and {} "
-              "Lindblad noise elements.".format(
-                  len(self.coherent_noise), len(self.lindblad_noise)))
+        print(
+            "The pulse contains: {} coherent noise elements and {} "
+            "Lindblad noise elements.".format(
+                len(self.coherent_noise), len(self.lindblad_noise)
+            )
+        )
         print()
         print("Ideal pulse:")
         print(self.ideal_pulse)
@@ -451,13 +485,16 @@ class Pulse():
             print("Lindblad noise:")
             for ele in self.lindblad_noise:
                 print(ele)
-        print("-----------------------------------"
-              "-----------------------------------")
+        print(
+            "-----------------------------------"
+            "-----------------------------------"
+        )
 
 
-class Drift():
+class Drift:
     """
-    The time independent drift Hamiltonian. Usually its the intrinsic
+    Representation of the time-independent drift Hamiltonian.
+    Usually its the intrinsic
     evolution of the quantum system that can not be tuned.
 
     Parameters
@@ -470,6 +507,7 @@ class Drift():
     qobj: list of :class:`qutip.Qobj`
         A list of the the drift Hamiltonians.
     """
+
     def __init__(self, qobj=None):
         if qobj is None:
             self.drift_hamiltonians = []
@@ -484,7 +522,7 @@ class Drift():
 
         Parameters
         ----------
-        qobj: :class:'qutip.Qobj'
+        qobj: :class:`qutip.Qobj`
             The collapse operator of the lindblad noise.
         targets: list
             target qubits of the collapse operator
@@ -510,7 +548,9 @@ class Drift():
         """
         if not self.drift_hamiltonians:
             self.drift_hamiltonians = [_EvoElement(None, None)]
-        qu_list = [QobjEvo(evo.get_qobj(dims)) for evo in self.drift_hamiltonians]
+        qu_list = [
+            QobjEvo(evo.get_qobj(dims)) for evo in self.drift_hamiltonians
+        ]
         return _merge_qobjevo(qu_list)
 
     def get_noisy_qobjevo(self, dims):
@@ -527,37 +567,47 @@ class Drift():
         """
         return self.get_ideal_qobjevo(dims), []
 
+    def get_full_tlist(self):
+        """
+        Drift has no tlist, this is just a place holder to keep it unified
+        with :class:`.Pulse`. It returns None.
+        """
+        return None
+
 
 def _find_common_tlist(qobjevo_list, tol=1.0e-10):
     """
     Find the common ``tlist`` of a list of :class:`qutip.QobjEvo`.
     """
-    all_tlists = [qu.tlist for qu in qobjevo_list
-                  if isinstance(qu, QobjEvo) and qu.tlist is not None]
+    all_tlists = [
+        qu.tlist
+        for qu in qobjevo_list
+        if isinstance(qu, QobjEvo) and qu.tlist is not None
+    ]
     if not all_tlists:
         return None
     full_tlist = np.unique(np.sort(np.hstack(all_tlists)))
     full_tlist = np.concatenate(
-        (full_tlist[:1], full_tlist[1:][np.diff(full_tlist) > tol]))
+        (full_tlist[:1], full_tlist[1:][np.diff(full_tlist) > tol])
+    )
     return full_tlist
 
-########################################################################
-# These functions are moved here from qutip_qip.device.processor.py
-########################################################################
-
 
 def _merge_qobjevo(qobjevo_list, full_tlist=None):
     """
     Combine a list of `:class:qutip.QobjEvo` into one,
     different tlist will be merged.
     """
-    # TODO This method can be eventually integrated into QobjEvo, for
-    # which a more thorough test is required
-
     # no qobjevo
     if not qobjevo_list:
         raise ValueError("qobjevo_list is empty.")
 
+    # In qutip5 this can be done automatically.
+    if parse_version(qutip.__version__) >= parse_version("5.dev"):
+        return sum(
+            [op for op in qobjevo_list if isinstance(op, (Qobj, QobjEvo))]
+        )
+
     if full_tlist is None:
         full_tlist = _find_common_tlist(qobjevo_list)
     spline_types_num = set()
@@ -579,7 +629,8 @@ def _merge_qobjevo(qobjevo_list, full_tlist=None):
         for j, ele in enumerate(qobjevo.ops):
             if isinstance(ele.coeff, np.ndarray):
                 new_coeff = _fill_coeff(
-                    ele.coeff, qobjevo.tlist, full_tlist, args)
+                    ele.coeff, qobjevo.tlist, full_tlist, args
+                )
                 qobjevo_list[i].ops[j].coeff = new_coeff
         qobjevo_list[i].tlist = full_tlist
 
@@ -591,7 +642,6 @@ def _fill_coeff(old_coeffs, old_tlist, full_tlist, args=None, tol=1.0e-10):
     """
     Make a step function coefficients compatible with a longer ``tlist`` by
     filling the empty slot with the nearest left value.
-
     The returned ``coeff`` always have the same size as the ``tlist``.
     If `step_func`, the last element is 0.
     """
@@ -606,18 +656,18 @@ def _fill_coeff(old_coeffs, old_tlist, full_tlist, args=None, tol=1.0e-10):
         for new_ind in range(new_n):
             t = full_tlist[new_ind]
             if old_tlist[0] - t > tol:
-                new_coeff[new_ind] = 0.
+                new_coeff[new_ind] = 0.0
                 continue
             if t - old_tlist[-1] > tol:
-                new_coeff[new_ind] = 0.
+                new_coeff[new_ind] = 0.0
                 continue
             # tol is required because of the floating-point error
-            if old_tlist[old_ind+1] <= t + tol:
+            if old_tlist[old_ind + 1] <= t + tol:
                 old_ind += 1
             new_coeff[new_ind] = old_coeffs[old_ind]
     else:
         sp = CubicSpline(old_tlist, old_coeffs)
         new_coeff = sp(full_tlist)
-        new_coeff *= (full_tlist <= old_tlist[-1])
-        new_coeff *= (full_tlist >= old_tlist[0])
+        new_coeff *= full_tlist <= old_tlist[-1]
+        new_coeff *= full_tlist >= old_tlist[0]
     return new_coeff
