@@ -1,3 +1,4 @@
+"""Importation and exportation of QASM circuits"""
 import re
 import os
 from itertools import chain
@@ -7,18 +8,23 @@ import warnings
 from math import pi
 import numpy as np
 
-from .operations.gates import gate_sequence_product
 from .circuit import QubitCircuit
-from .operations.gates import controlled_gate, qasmu_gate, rz, snot
+from .operations import (
+    controlled_gate,
+    qasmu_gate,
+    rz,
+    snot,
+    gate_sequence_product,
+)
 
 
 __all__ = ["read_qasm", "save_qasm", "print_qasm", "circuit_to_qasm_str"]
 
 
 class QasmGate:
-    '''
+    """
     Class which stores the gate definitions as specified in the QASM file.
-    '''
+    """
 
     def __init__(self, name, gate_args, gate_regs):
         self.name = name
@@ -28,25 +34,25 @@ class QasmGate:
 
 
 def _get_qiskit_gates():
-    '''
+    """
     Create and return a dictionary containing custom gates needed
     for "qiskit" mode. These include a subset of gates usually defined
     in the file "qelib1.inc".
 
     Returns a dictionary mapping gate names to QuTiP gates.
-    '''
+    """
 
     def u2(args):
-        return qasmu_gate([np.pi/2, args[0], args[1]])
+        return qasmu_gate([np.pi / 2, args[0], args[1]])
 
     def id():
         return qasmu_gate([0, 0, 0])
 
     def sdg():
-        return rz(-1 * np.pi/2)
+        return rz(-1 * np.pi / 2)
 
     def tdg():
-        return rz(-1 * np.pi/4)
+        return rz(-1 * np.pi / 4)
 
     def cu3(args):
         return controlled_gate(qasmu_gate(args))
@@ -58,7 +64,7 @@ def _get_qiskit_gates():
 
 
 def _tokenize_line(command):
-    '''
+    """
     Tokenize (break into several parts a string of) a single line of QASM code.
 
     Parameters
@@ -70,7 +76,7 @@ def _tokenize_line(command):
     -------
     tokens : list of str
         The tokens (parts) corresponding to the qasm line taken as input.
-    '''
+    """
 
     # for gates without arguments
     if "(" not in command:
@@ -82,9 +88,11 @@ def _tokenize_line(command):
         # for classically controlled gates with arguments
         if groups:
             tokens = ["if", "(", groups.group(1), ")"]
-            tokens_gate = _tokenize_line("{} ({}) {}".format(groups.group(2),
-                                                             groups.group(3),
-                                                             groups.group(4)))
+            tokens_gate = _tokenize_line(
+                "{} ({}) {}".format(
+                    groups.group(2), groups.group(3), groups.group(4)
+                )
+            )
             tokens += tokens_gate
         # for classically controlled gates without arguments
         else:
@@ -109,7 +117,7 @@ def _tokenize_line(command):
 
 
 def _tokenize(token_cmds):
-    '''
+    """
     Tokenize QASM code for processing, i.e. break it into several parts.
 
     Parameters
@@ -121,7 +129,7 @@ def _tokenize(token_cmds):
     -------
     tokens : list of (list of str)
         List of tokens corresponding to each QASM line taken as input.
-    '''
+    """
 
     processed_commands = []
 
@@ -143,11 +151,11 @@ def _tokenize(token_cmds):
 
 
 def _gate_processor(command):
-    '''
+    """
     Process tokens for a gate call statement separating them into args and regs.
     Processes tokens from a "gate call" (e.g. rx(pi) q[0]) and returns the
     tokens for the arguments and registers separately.
-    '''
+    """
 
     gate_args = []
     gate_regs = []
@@ -158,8 +166,8 @@ def _gate_processor(command):
     if "(" in tokens and ")" in tokens:
         bopen = tokens.index("(")
         bclose = tokens.index(")")
-        gate_args = tokens[bopen+1:bclose]
-        reg_start = bclose+1
+        gate_args = tokens[bopen + 1 : bclose]
+        reg_start = bclose + 1
 
     # extract registers
     gate_regs = tokens[reg_start:]
@@ -168,9 +176,9 @@ def _gate_processor(command):
 
 
 class QasmProcessor:
-    '''
+    """
     Class which holds variables used in processing QASM code.
-    '''
+    """
 
     def __init__(self, commands, mode="qiskit", version="2.0"):
         self.qubit_regs = {}
@@ -183,26 +191,51 @@ class QasmProcessor:
         self.predefined_gates = set(["CX", "U"])
 
         if self.mode == "qiskit":
-            self.qiskitgates = set(["u3", "u2", "u1", "cx",  "id", "x", "y",
-                                    "z", "h", "s", "sdg", "t", "tdg", "rx",
-                                    "ry", "rz", "cz", "cy", "ch", "ccx", "crz",
-                                    "cu1", "cu3"])
-            self.predefined_gates = self.predefined_gates.union(self.qiskitgates)
+            self.qiskitgates = set(
+                [
+                    "u3",
+                    "u2",
+                    "u1",
+                    "cx",
+                    "id",
+                    "x",
+                    "y",
+                    "z",
+                    "h",
+                    "s",
+                    "sdg",
+                    "t",
+                    "tdg",
+                    "rx",
+                    "ry",
+                    "rz",
+                    "cz",
+                    "cy",
+                    "ch",
+                    "ccx",
+                    "crz",
+                    "cu1",
+                    "cu3",
+                ]
+            )
+            self.predefined_gates = self.predefined_gates.union(
+                self.qiskitgates
+            )
 
         self.gate_names = deepcopy(self.predefined_gates)
         for gate in self.predefined_gates:
-            self.qasm_gates[gate] = QasmGate("U",
-                                             ["alpha", "beta", "gamma"],
-                                             ["q"])
+            self.qasm_gates[gate] = QasmGate(
+                "U", ["alpha", "beta", "gamma"], ["q"]
+            )
         self.commands = commands
 
     def _process_includes(self):
-        '''
+        """
         QASM allows for code to be specified in additional files with the
         ".inc" extension, especially to specify gate definitions in terms of
         the built-in gates. Process into tokens all the
         additional files and insert it into previously processed list.
-        '''
+        """
 
         prev_index = 0
         expanded_commands = []
@@ -219,15 +252,18 @@ class QasmProcessor:
 
             if os.path.exists(filename):
                 with open(filename, "r") as f:
-                    qasm_lines = [line.strip() for line
-                                  in f.read().splitlines()]
-                    qasm_lines = list(filter(
-                                      lambda x: x[:2] != "//" and x != "",
-                                      qasm_lines))
+                    qasm_lines = [
+                        line.strip() for line in f.read().splitlines()
+                    ]
+                    qasm_lines = list(
+                        filter(lambda x: x[:2] != "//" and x != "", qasm_lines)
+                    )
 
-                    expanded_commands = (expanded_commands
-                                         + command[prev_index:curr_index]
-                                         + _tokenize(qasm_lines))
+                    expanded_commands = (
+                        expanded_commands
+                        + command[prev_index:curr_index]
+                        + _tokenize(qasm_lines)
+                    )
                     prev_index = curr_index + 1
             else:
                 raise ValueError(command[1] + ": such a file does not exist")
@@ -236,10 +272,10 @@ class QasmProcessor:
         self.commands = expanded_commands
 
     def _initialize_pass(self):
-        '''
+        """
         Passes through the tokenized commands, create QasmGate objects for
         each user-defined gate, process register declarations.
-        '''
+        """
 
         gate_defn_mode = False
         open_bracket_mode = False
@@ -260,10 +296,14 @@ class QasmProcessor:
                     raise SyntaxError("QASM: incorrect bracket formatting")
                 elif command[0] == "}":
                     if not curr_gate.gates_inside:
-                        raise NotImplementedError("QASM: opaque gate {} are  \
+                        raise NotImplementedError(
+                            "QASM: opaque gate {} are  \
                                                    not allowed, please define \
                                                    or omit \
-                                                   them".format(curr_gate.name))
+                                                   them".format(
+                                curr_gate.name
+                            )
+                        )
                     open_bracket_mode = False
                     self.gate_names.add(curr_gate.name)
                     self.qasm_gates[curr_gate.name] = curr_gate
@@ -272,9 +312,7 @@ class QasmProcessor:
                     name = command[0]
                     gate_args, gate_regs = _gate_processor(command)
                     gate_added = self.qasm_gates[name]
-                    curr_gate.gates_inside.append([name,
-                                                   gate_args,
-                                                   gate_regs])
+                    curr_gate.gates_inside.append([name, gate_args, gate_regs])
             elif command[0] == "gate":
                 gate_name = command[1]
                 gate_args, gate_regs = _gate_processor(command[1:])
@@ -286,9 +324,9 @@ class QasmProcessor:
                 if groups:
                     qubit_name = groups.group(1)
                     num_regs = int(groups.group(2))
-                    self.qubit_regs[qubit_name] = list(range(
-                                                self.num_qubits,
-                                                self.num_qubits + num_regs))
+                    self.qubit_regs[qubit_name] = list(
+                        range(self.num_qubits, self.num_qubits + num_regs)
+                    )
                     self.num_qubits += num_regs
                 else:
                     raise SyntaxError("QASM: incorrect bracket formatting")
@@ -298,14 +336,16 @@ class QasmProcessor:
                 if groups:
                     cbit_name = groups.group(1)
                     num_regs = int(groups.group(2))
-                    self.cbit_regs[cbit_name] = list(range(self.num_cbits,
-                                                     self.num_cbits + num_regs))
+                    self.cbit_regs[cbit_name] = list(
+                        range(self.num_cbits, self.num_cbits + num_regs)
+                    )
                     self.num_cbits += num_regs
                 else:
                     raise SyntaxError("QASM: incorrect bracket formatting")
             elif command[0] == "reset":
-                raise NotImplementedError(("QASM: reset functionality "
-                                           "is not supported."))
+                raise NotImplementedError(
+                    ("QASM: reset functionality " "is not supported.")
+                )
             elif command[0] in ["barrier", "include"]:
                 continue
             else:
@@ -318,7 +358,7 @@ class QasmProcessor:
         self.commands = [self.commands[i] for i in unprocessed]
 
     def _custom_gate(self, qc_temp, gate_call):
-        '''
+        """
         Recursively process a custom-defined gate with specified arguments
         to produce a dummy circuit with all the gates in the custom-defined
         gate.
@@ -330,7 +370,7 @@ class QasmProcessor:
             temporary circuit to process custom gate
         gate_call: list of str
             tokens corresponding to gate signature/call
-        '''
+        """
 
         gate_name, args, regs = gate_call
         gate = self.qasm_gates[gate_name]
@@ -349,11 +389,15 @@ class QasmProcessor:
             name, com_args, com_regs = call
 
             for arg, real_arg in args_map.items():
-                com_args = [command.replace(arg.strip(), str(real_arg))
-                            for command in com_args]
+                com_args = [
+                    command.replace(arg.strip(), str(real_arg))
+                    for command in com_args
+                ]
             for reg, real_reg in regs_map.items():
-                com_regs = [command.replace(reg.strip(), str(real_reg))
-                            for command in com_regs]
+                com_regs = [
+                    command.replace(reg.strip(), str(real_reg))
+                    for command in com_regs
+                ]
             com_args = [eval(arg) for arg in com_args]
 
             if name in self.predefined_gates:
@@ -364,7 +408,7 @@ class QasmProcessor:
                 self._custom_gate(qc_temp, [name, com_args, com_regs])
 
     def _regs_processor(self, regs, reg_type):
-        '''
+        """
         Process register tokens: map them to the :class:`.QubitCircuit` indices
         of the respective registers.
 
@@ -383,7 +427,7 @@ class QasmProcessor:
                 list of register sets to which circuit operations
                 are applied.
 
-        '''
+        """
 
         # turns messy tokens into appropriate form
         # ['q', 'p', '[', '0', ']'] -> ['q', 'p[0]']
@@ -439,8 +483,10 @@ class QasmProcessor:
                 if len(qubits) == len(cbits):
                     return zip(qubits, cbits)
                 else:
-                    raise ValueError("QASM: qubit and cbit \
-                                     register sizes are different")
+                    raise ValueError(
+                        "QASM: qubit and cbit \
+                                     register sizes are different"
+                    )
         else:
             # processes gate tokens to create sets of registers to
             # which the gates are applied.
@@ -460,14 +506,28 @@ class QasmProcessor:
                     expand = len(qubit)
                 new_regs.append(qubit)
             if expand:
-                return zip(*list(map(
-                        lambda x: x if isinstance(x, list) else [x] * expand,
-                        new_regs)))
+                return zip(
+                    *list(
+                        map(
+                            lambda x: x
+                            if isinstance(x, list)
+                            else [x] * expand,
+                            new_regs,
+                        )
+                    )
+                )
             else:
                 return [new_regs]
 
-    def _add_qiskit_gates(self, qc, name, regs, args=None,
-                          classical_controls=None, control_value=None):
+    def _add_qiskit_gates(
+        self,
+        qc,
+        name,
+        regs,
+        args=None,
+        classical_controls=None,
+        control_value=None,
+    ):
         """
         Add any gates that are pre-defined in qiskit-style exported
         qasm file with included "qelib1.inc".
@@ -492,70 +552,133 @@ class QasmProcessor:
             (i.e. all classical controls are 1).
         """
 
-        gate_name_map_1q = {"x": "X", "y": "Y", "z": "Z", "h": "SNOT",
-                            "t": "T", "s": "S", "sdg": "sdg", "tdg": "tdg",
-                            "rx": "RX", "ry": "RY", "rz": "RZ"}
+        gate_name_map_1q = {
+            "x": "X",
+            "y": "Y",
+            "z": "Z",
+            "h": "SNOT",
+            "t": "T",
+            "s": "S",
+            "sdg": "sdg",
+            "tdg": "tdg",
+            "rx": "RX",
+            "ry": "RY",
+            "rz": "RZ",
+        }
         if len(args) == 0:
             args = None
         elif len(args) == 1:
             args = args[0]
 
         if name == "u3":
-            qc.add_gate("QASMU", targets=regs[0], arg_value=args,
-                        classical_controls=classical_controls,
-                        control_value=control_value)
+            qc.add_gate(
+                "QASMU",
+                targets=regs[0],
+                arg_value=args,
+                classical_controls=classical_controls,
+                control_value=control_value,
+            )
         elif name == "u2":
-            qc.add_gate("u2", targets=regs[0], arg_value=args,
-                        classical_controls=classical_controls,
-                        control_value=control_value)
+            qc.add_gate(
+                "u2",
+                targets=regs[0],
+                arg_value=args,
+                classical_controls=classical_controls,
+                control_value=control_value,
+            )
         elif name == "u1":
-            qc.add_gate("RZ", targets=regs[0], arg_value=args,
-                        classical_controls=classical_controls,
-                        control_value=control_value)
+            qc.add_gate(
+                "RZ",
+                targets=regs[0],
+                arg_value=args,
+                classical_controls=classical_controls,
+                control_value=control_value,
+            )
         elif name == "cz":
-            qc.add_gate("CZ", targets=regs[1], controls=regs[0],
-                        classical_controls=classical_controls,
-                        control_value=control_value)
+            qc.add_gate(
+                "CZ",
+                targets=regs[1],
+                controls=regs[0],
+                classical_controls=classical_controls,
+                control_value=control_value,
+            )
         elif name == "cy":
-            qc.add_gate("CY", targets=regs[1], controls=regs[0],
-                        classical_controls=classical_controls,
-                        control_value=control_value)
+            qc.add_gate(
+                "CY",
+                targets=regs[1],
+                controls=regs[0],
+                classical_controls=classical_controls,
+                control_value=control_value,
+            )
         elif name == "ch":
-            qc.add_gate("ch", targets=regs,
-                        classical_controls=classical_controls,
-                        control_value=control_value)
+            qc.add_gate(
+                "ch",
+                targets=regs,
+                classical_controls=classical_controls,
+                control_value=control_value,
+            )
         elif name == "ccx":
-            qc.add_gate("TOFFOLI", targets=regs[2], controls=regs[:2],
-                        classical_controls=classical_controls,
-                        control_value=control_value)
+            qc.add_gate(
+                "TOFFOLI",
+                targets=regs[2],
+                controls=regs[:2],
+                classical_controls=classical_controls,
+                control_value=control_value,
+            )
         elif name == "crz":
-            qc.add_gate("CRZ", targets=regs[1], controls=regs[0],
-                        arg_value=args,
-                        classical_controls=classical_controls,
-                        control_value=control_value)
+            qc.add_gate(
+                "CRZ",
+                targets=regs[1],
+                controls=regs[0],
+                arg_value=args,
+                classical_controls=classical_controls,
+                control_value=control_value,
+            )
         elif name == "cu1":
-            qc.add_gate("CPHASE", targets=regs[1], controls=regs[0],
-                        arg_value=args,
-                        classical_controls=classical_controls,
-                        control_value=control_value)
+            qc.add_gate(
+                "CPHASE",
+                targets=regs[1],
+                controls=regs[0],
+                arg_value=args,
+                classical_controls=classical_controls,
+                control_value=control_value,
+            )
         elif name == "cu3":
-            qc.add_gate("cu3", targets=[regs[0], regs[1]],
-                        arg_value=args,
-                        classical_controls=classical_controls,
-                        control_value=control_value)
+            qc.add_gate(
+                "cu3",
+                targets=[regs[0], regs[1]],
+                arg_value=args,
+                classical_controls=classical_controls,
+                control_value=control_value,
+            )
         if name == "cx":
-            qc.add_gate("CNOT",  targets=int(regs[1]), controls=int(regs[0]),
-                        classical_controls=classical_controls,
-                        control_value=control_value)
+            qc.add_gate(
+                "CNOT",
+                targets=int(regs[1]),
+                controls=int(regs[0]),
+                classical_controls=classical_controls,
+                control_value=control_value,
+            )
         elif name in gate_name_map_1q:
             if args == []:
                 args = None
-            qc.add_gate(gate_name_map_1q[name], targets=regs[0], arg_value=args,
-                        classical_controls=classical_controls,
-                        control_value=control_value)
+            qc.add_gate(
+                gate_name_map_1q[name],
+                targets=regs[0],
+                arg_value=args,
+                classical_controls=classical_controls,
+                control_value=control_value,
+            )
 
-    def _add_predefined_gates(self, qc, name, com_regs, com_args,
-                              classical_controls=None, control_value=None):
+    def _add_predefined_gates(
+        self,
+        qc,
+        name,
+        com_regs,
+        com_args,
+        classical_controls=None,
+        control_value=None,
+    ):
         """
         Add any gates that are pre-defined and/or inbuilt
         in our circuit.
@@ -581,24 +704,35 @@ class QasmProcessor:
         """
 
         if name == "CX":
-            qc.add_gate("CNOT",
-                        targets=int(com_regs[1]),
-                        controls=int(com_regs[0]),
-                        classical_controls=classical_controls,
-                        control_value=control_value)
+            qc.add_gate(
+                "CNOT",
+                targets=int(com_regs[1]),
+                controls=int(com_regs[0]),
+                classical_controls=classical_controls,
+                control_value=control_value,
+            )
         elif name == "U":
-            qc.add_gate("QASMU",
-                        targets=int(com_regs[0]),
-                        arg_value=[float(arg) for arg in com_args],
-                        classical_controls=classical_controls,
-                        control_value=control_value)
+            qc.add_gate(
+                "QASMU",
+                targets=int(com_regs[0]),
+                arg_value=[float(arg) for arg in com_args],
+                classical_controls=classical_controls,
+                control_value=control_value,
+            )
         elif name in self.qiskitgates and self.mode == "qiskit":
-            self._add_qiskit_gates(qc, name, com_regs, com_args,
-                                   classical_controls, control_value)
+            self._add_qiskit_gates(
+                qc, name, com_regs, com_args, classical_controls, control_value
+            )
 
-    def _gate_add(self, qc, command, custom_gates,
-                  classical_controls=None, control_value=None):
-        '''
+    def _gate_add(
+        self,
+        qc,
+        command,
+        custom_gates,
+        classical_controls=None,
+        control_value=None,
+    ):
+        """
         Add gates to :class:`.QubitCircuit` from processed tokens,
         define custom gates if necessary.
 
@@ -618,7 +752,7 @@ class QasmProcessor:
             If not specified, then the value is interpreted to be
             2 ** len(classical_controls) - 1
             (i.e. all classical controls are 1).
-        '''
+        """
 
         args, regs = _gate_processor(command)
         reg_set = self._regs_processor(regs, "gate")
@@ -629,12 +763,15 @@ class QasmProcessor:
             gate_name = "{}".format(command[0])
 
         # creates custom-gate (if required) using gate defn and provided args
-        if (command[0] not in self.predefined_gates
-                and command[0] not in custom_gates):
+        if (
+            command[0] not in self.predefined_gates
+            and command[0] not in custom_gates
+        ):
             n = len(reg_set[0])
             qc_temp = QubitCircuit(n)
-            self._custom_gate(qc_temp,
-                              [command[0], args, [str(i) for i in range(n)]])
+            self._custom_gate(
+                qc_temp, [command[0], args, [str(i) for i in range(n)]]
+            )
             unitary_mat = gate_sequence_product(qc_temp.propagators())
             custom_gates[gate_name] = unitary_mat
 
@@ -646,25 +783,28 @@ class QasmProcessor:
             if command[0] in self.predefined_gates:
                 args = [eval(arg) for arg in args]
                 self._add_predefined_gates(
-                                    qc,
-                                    command[0],
-                                    regs,
-                                    args,
-                                    classical_controls=classical_controls,
-                                    control_value=control_value)
+                    qc,
+                    command[0],
+                    regs,
+                    args,
+                    classical_controls=classical_controls,
+                    control_value=control_value,
+                )
             else:
                 if not isinstance(regs, list):
                     regs = [regs]
-                qc.add_gate(gate_name,
-                            targets=regs,
-                            classical_controls=classical_controls,
-                            control_value=control_value)
+                qc.add_gate(
+                    gate_name,
+                    targets=regs,
+                    classical_controls=classical_controls,
+                    control_value=control_value,
+                )
 
     def _final_pass(self, qc):
-        '''
+        """
         Take a blank circuit, add all the gates and measurements specified
         by QASM.
-        '''
+        """
 
         custom_gates = {}
         if self.mode == "qiskit":
@@ -678,24 +818,32 @@ class QasmProcessor:
                 # adds measurement to the QubitCircuit
                 reg_set = self._regs_processor(command[1:], "measure")
                 for regs in reg_set:
-                    qc.add_measurement("M", targets=[regs[0]],
-                                       classical_store=regs[1])
+                    qc.add_measurement(
+                        "M", targets=[regs[0]], classical_store=regs[1]
+                    )
             elif command[0] == "if":
-                warnings.warn(("Information about individual registers"
-                              " is not preserved in QubitCircuit"))
+                warnings.warn(
+                    (
+                        "Information about individual registers"
+                        " is not preserved in QubitCircuit"
+                    )
+                )
                 # adds classically controlled gates to the QubitCircuit
                 cbit_reg, control_value = command[2].split("==")
                 cbit_inds = self.cbit_regs[cbit_reg]
                 control_value = int(control_value)
-                self._gate_add(qc, command[4:], custom_gates,
-                               cbit_inds, control_value)
+                self._gate_add(
+                    qc, command[4:], custom_gates, cbit_inds, control_value
+                )
             else:
-                err = "QASM: {} is not a valid QASM command.".format(command[0])
+                err = "QASM: {} is not a valid QASM command.".format(
+                    command[0]
+                )
                 raise SyntaxError(err)
 
 
 def read_qasm(qasm_input, mode="qiskit", version="2.0", strmode=False):
-    '''
+    """
     Read OpenQASM intermediate representation
     (https://github.com/Qiskit/openqasm) and return
     a :class:`.QubitCircuit` and state inputs as specified in the
@@ -721,7 +869,7 @@ def read_qasm(qasm_input, mode="qiskit", version="2.0", strmode=False):
     -------
     qc : :class:`.QubitCircuit`
         Returns a :class:`.QubitCircuit` object specified in the QASM file.
-    '''
+    """
 
     if strmode:
         qasm_lines = qasm_input.splitlines()
@@ -735,8 +883,10 @@ def read_qasm(qasm_input, mode="qiskit", version="2.0", strmode=False):
     qasm_lines = list(filter(lambda x: x[:2] != "//" and x != "", qasm_lines))
 
     if version != "2.0":
-        raise NotImplementedError("QASM: Only OpenQASM 2.0 \
-                                  is currently supported.")
+        raise NotImplementedError(
+            "QASM: Only OpenQASM 2.0 \
+                                  is currently supported."
+        )
 
     if qasm_lines.pop(0) != "OPENQASM 2.0;":
         raise SyntaxError("QASM: File does not contain QASM 2.0 header")
@@ -767,11 +917,11 @@ _GATE_NAME_TO_QASM_NAME = {
     "T": "t",
     "CRZ": "crz",
     "CNOT": "cx",
-    "TOFFOLI": "ccx"
+    "TOFFOLI": "ccx",
 }
 
 
-class QasmOutput():
+class QasmOutput:
     """
     Class for QASM export.
 
@@ -787,7 +937,7 @@ class QasmOutput():
         self.gate_name_map = deepcopy(_GATE_NAME_TO_QASM_NAME)
 
     def output(self, line="", n=0):
-        '''
+        """
         Pipe QASM output string to QasmOutput's lines variable.
 
         Parameters
@@ -796,32 +946,32 @@ class QasmOutput():
             string to be appended to QASM output.
         n: int, optional
             number of blank lines to be appended to QASM output.
-        '''
+        """
 
         if line:
             self.lines.append(line)
         self.lines = self.lines + [""] * n
 
     def _flush(self):
-        '''
+        """
         Resets QasmOutput variables.
-        '''
+        """
 
         self.lines = []
         self.gate_name_map = deepcopy(_GATE_NAME_TO_QASM_NAME)
 
     def _qasm_str(self, q_name, q_controls, q_targets, q_args):
-        '''
+        """
         Returns QASM string for gate definition or gate application given
         name, registers, arguments.
-        '''
+        """
 
         if not q_controls:
             q_controls = []
         q_regs = q_controls + q_targets
 
         if isinstance(q_targets[0], int):
-            q_regs = ",".join(['q[{}]'.format(reg) for reg in q_regs])
+            q_regs = ",".join(["q[{}]".format(reg) for reg in q_regs])
         else:
             q_regs = ",".join(q_regs)
 
@@ -833,7 +983,7 @@ class QasmOutput():
             return "{} {};".format(q_name, q_regs)
 
     def _qasm_defn_from_resolved(self, curr_gate, gates_lst):
-        '''
+        """
         Resolve QASM definition of QuTiP gate in terms of component gates.
 
         Parameters
@@ -842,10 +992,10 @@ class QasmOutput():
             QuTiP gate which needs to be resolved into component gates.
         gates_lst: list of :class:`.Gate`
             list of gate that constitute QASM definition of self.
-        '''
+        """
 
         forbidden_gates = ["GLOBALPHASE", "PHASEGATE"]
-        reg_map = ['a', 'b', 'c']
+        reg_map = ["a", "b", "c"]
 
         q_controls = None
         if curr_gate.controls:
@@ -855,30 +1005,40 @@ class QasmOutput():
         if curr_gate.arg_value:
             arg_name = "theta"
 
-        self.output("gate {} {{".format(self._qasm_str(curr_gate.name.lower(),
-                                                       q_controls,
-                                                       q_targets,
-                                                       arg_name)[:-1]))
+        self.output(
+            "gate {} {{".format(
+                self._qasm_str(
+                    curr_gate.name.lower(), q_controls, q_targets, arg_name
+                )[:-1]
+            )
+        )
 
         for gate in gates_lst:
             if gate.name in self.gate_name_map:
                 gate.targets = [reg_map[i] for i in gate.targets]
                 if gate.controls:
                     gate.controls = [reg_map[i] for i in gate.controls]
-                self.output(self._qasm_str(
-                                self.gate_name_map[gate.name],
-                                gate.controls,
-                                gate.targets,
-                                gate.arg_value))
+                self.output(
+                    self._qasm_str(
+                        self.gate_name_map[gate.name],
+                        gate.controls,
+                        gate.targets,
+                        gate.arg_value,
+                    )
+                )
             elif gate.name in forbidden_gates:
                 continue
             else:
-                raise ValueError(("The given resolved gate {} cannot be defined"
-                                  " in QASM format").format(curr_gate.name))
+                raise ValueError(
+                    (
+                        "The given resolved gate {} cannot be defined"
+                        " in QASM format"
+                    ).format(curr_gate.name)
+                )
         self.output("}")
 
     def _qasm_defn_resolve(self, gate):
-        '''
+        """
         Resolve QASM definition of QuTiP gate if possible.
 
         Parameters
@@ -886,7 +1046,7 @@ class QasmOutput():
         gate: :class:`.Gate`
             QuTiP gate which needs to be resolved into component gates.
 
-        '''
+        """
 
         qc = QubitCircuit(3)
         gates_lst = []
@@ -900,14 +1060,14 @@ class QasmOutput():
         self.gate_name_map[gate.name] = gate.name.lower()
 
     def _qasm_defns(self, gate):
-        '''
+        """
         Define QASM gates for QuTiP gates that do not have QASM counterparts.
 
         Parameters
         ----------
         gate: :class:`.Gate`
             QuTiP gate which needs to be defined in QASM format.
-        '''
+        """
 
         if gate.name == "CRY":
             gate_def = "gate cry(theta) a,b { cu3(theta,0,0) a,b; }"
@@ -930,14 +1090,14 @@ class QasmOutput():
         self.gate_name_map[gate.name] = gate.name.lower()
 
     def qasm_name(self, gate_name):
-        '''
+        """
         Return QASM gate name for corresponding QuTiP gate.
 
         Parameters
         ----------
         gate_name: str
             QuTiP gate name.
-        '''
+        """
 
         if gate_name in self.gate_name_map:
             return self.gate_name_map[gate_name]
@@ -945,26 +1105,26 @@ class QasmOutput():
             return None
 
     def is_defined(self, gate_name):
-        '''
+        """
         Check if QASM gate definition exists for QuTiP gate.
 
         Parameters
         ----------
         gate_name: str
             QuTiP gate name.
-        '''
+        """
 
         return gate_name in self.gate_name_map
 
     def _qasm_output(self, qc):
-        '''
+        """
         QASM output handler.
 
         Parameters
         ----------
         qc : :class:`.QubitCircuit`
             circuit object to produce QASM output for.
-        '''
+        """
 
         self._flush()
 
@@ -973,8 +1133,10 @@ class QasmOutput():
         if self.version == "2.0":
             self.output("OPENQASM 2.0;")
         else:
-            raise NotImplementedError("QASM: Only OpenQASM 2.0 \
-                                      is currently supported.")
+            raise NotImplementedError(
+                "QASM: Only OpenQASM 2.0 \
+                                      is currently supported."
+            )
 
         self.output('include "qelib1.inc";', 1)
 
@@ -984,14 +1146,14 @@ class QasmOutput():
 
 
 def print_qasm(qc):
-    '''
+    """
     Print QASM output of circuit object.
 
     Parameters
     ----------
     qc : :class:`.QubitCircuit`
         circuit object to produce QASM output for.
-    '''
+    """
 
     qasm_out = QasmOutput("2.0")
     lines = qasm_out._qasm_output(qc)
@@ -1000,7 +1162,7 @@ def print_qasm(qc):
 
 
 def circuit_to_qasm_str(qc):
-    '''
+    """
     Return QASM output of circuit object as string
 
     Parameters
@@ -1012,7 +1174,7 @@ def circuit_to_qasm_str(qc):
     -------
     output: str
         string corresponding to QASM output.
-    '''
+    """
 
     qasm_out = QasmOutput("2.0")
     lines = qasm_out._qasm_output(qc)
@@ -1023,14 +1185,14 @@ def circuit_to_qasm_str(qc):
 
 
 def save_qasm(qc, file_loc):
-    '''
+    """
     Save QASM output of circuit object to file.
 
     Parameters
     ----------
     qc : :class:`.QubitCircuit`
         circuit object to produce QASM output for.
-    '''
+    """
 
     qasm_out = QasmOutput("2.0")
     lines = qasm_out._qasm_output(qc)
