@@ -1,50 +1,23 @@
-# This file is part of QuTiP: Quantum Toolbox in Python.
-#
-#    Copyright (c) 2011 and later, Paul D. Nation and Robert J. Johansson.
-#    All rights reserved.
-#
-#    Redistribution and use in source and binary forms, with or without
-#    modification, are permitted provided that the following conditions are
-#    met:
-#
-#    1. Redistributions of source code must retain the above copyright notice,
-#       this list of conditions and the following disclaimer.
-#
-#    2. Redistributions in binary form must reproduce the above copyright
-#       notice, this list of conditions and the following disclaimer in the
-#       documentation and/or other materials provided with the distribution.
-#
-#    3. Neither the name of the QuTiP: Quantum Toolbox in Python nor the names
-#       of its contributors may be used to endorse or promote products derived
-#       from this software without specific prior written permission.
-#
-#    THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
-#    "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
-#    LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
-#    PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
-#    HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
-#    SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
-#    LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
-#    DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
-#    THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
-#    (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
-#    OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
-###############################################################################
-
-
 import pytest
+import os
+import shutil
 import numpy as np
 from pathlib import Path
 
-from qutip_qip.operations import gates
-from qutip.operators import identity
+
 from qutip_qip.circuit import (
-    QubitCircuit, CircuitSimulator, Gate, Measurement, _ctrl_gates,
-    _single_qubit_gates, _swap_like, _toffoli_like, _fredkin_like, _para_gates)
+    QubitCircuit, CircuitSimulator, Measurement)
 from qutip import (tensor, Qobj, ptrace, rand_ket, fock_dm, basis,
-                   rand_dm, bell_state, ket2dm)
+                   rand_dm, bell_state, ket2dm, identity, sigmax)
 from qutip_qip.qasm import read_qasm
-from qutip_qip.operations.gates import gate_sequence_product
+from qutip_qip.operations import (
+    Gate, gates, gate_sequence_product,
+    _ctrl_gates, _single_qubit_gates, _swap_like, _toffoli_like, _fredkin_like,
+    _para_gates
+)
+
+from qutip_qip._decompose.decompose_single_qubit_gate import _ZYZ_rotation
+
 import qutip as qp
 
 
@@ -257,10 +230,66 @@ class TestQubitCircuit:
             # Single control
             pytest.raises(ValueError, qc.add_gate, gate, [0], [1])
 
+        dummy_gate1 = Gate("DUMMY1")
+        inds = [1, 3, 4, 6]
+        qc.add_gate(dummy_gate1, index=inds)
+
+        # Test adding gates at multiple (sorted) indices at once.
+        # NOTE: Every insertion shifts the indices in the original list of
+        #       gates by an additional position to the right.
+        expected_gate_names = [
+            'CNOT',     # 0
+            'DUMMY1',   # 1
+            'SWAP',     # 2
+            'TOFFOLI',  # 3
+            'DUMMY1',   # 4
+            'SWAP',     # 5
+            'DUMMY1',   # 6
+            'SNOT',     # 7
+            'RY',       # 8
+            'DUMMY1',   # 9
+            'RY',       # 10
+        ]
+        actual_gate_names = [gate.name for gate in qc.gates]
+        assert actual_gate_names == expected_gate_names
+
+        dummy_gate2 = Gate("DUMMY2")
+        inds = [11, 0]
+        qc.add_gate(dummy_gate2, index=inds)
+
+        # Test adding gates at multiple (unsorted) indices at once.
+        expected_gate_names = [
+            'DUMMY2',   # 0
+            'CNOT',     # 1
+            'DUMMY1',   # 2
+            'SWAP',     # 3
+            'TOFFOLI',  # 4
+            'DUMMY1',   # 5
+            'SWAP',     # 6
+            'DUMMY1',   # 7
+            'SNOT',     # 8
+            'RY',       # 9
+            'DUMMY1',   # 10
+            'RY',       # 11
+            'DUMMY2',   # 12
+        ]
+        actual_gate_names = [gate.name for gate in qc.gates]
+        assert actual_gate_names == expected_gate_names
+
     def test_add_circuit(self):
         """
         Addition of a circuit to a `QubitCircuit`
         """
+
+        def customer_gate1(arg_values):
+            mat = np.zeros((4, 4), dtype=np.complex128)
+            mat[0, 0] = mat[1, 1] = 1.
+            mat[2:4, 2:4] = gates.rx(arg_values)
+            return Qobj(mat, dims=[[2, 2], [2, 2]])
+
+        qc = QubitCircuit(6)
+        qc.user_gates = {"CTRLRX": customer_gate1}
+
         qc = QubitCircuit(6)
         qc.add_gate("CNOT", targets=[1], controls=[0])
         test_gate = Gate("SWAP", targets=[1, 4])
@@ -270,6 +299,7 @@ class TestQubitCircuit:
         qc.add_gate(test_gate, index=[3])
         qc.add_measurement("M0", targets=[0], classical_store=[1])
         qc.add_1q_gate("RY", start=4, end=5, arg_value=1.570796)
+        qc.add_gate("CTRLRX", targets=[1, 2], arg_value=np.pi/2)
 
         qc1 = QubitCircuit(6)
 
@@ -278,6 +308,9 @@ class TestQubitCircuit:
         # Test if all gates and measurements are added
         assert len(qc1.gates) == len(qc.gates)
 
+        # Test if the definitions of user gates are added
+        assert qc1.user_gates == qc.user_gates
+
         for i in range(len(qc1.gates)):
             assert (qc1.gates[i].name
                     == qc.gates[i].name)
@@ -313,6 +346,10 @@ class TestQubitCircuit:
                 assert (qc2.gates[i].controls[0]
                         == qc.gates[i].controls[0]+2)
 
+        # Test exception when the operators to be added are not gates or measurements
+        qc.gates[-1] = 0
+        pytest.raises(TypeError, qc2.add_circuit, qc)
+
     def test_add_state(self):
         """
         Addition of input and output states to a circuit.
@@ -486,7 +523,7 @@ class TestQubitCircuit:
         def customer_gate1(arg_values):
             mat = np.zeros((4, 4), dtype=np.complex128)
             mat[0, 0] = mat[1, 1] = 1.
-            mat[2:4, 2:4] = gates.rx(arg_values)
+            mat[2:4, 2:4] = gates.rx(arg_values).full()
             return Qobj(mat, dims=[[2, 2], [2, 2]])
 
         def customer_gate2():
@@ -501,15 +538,15 @@ class TestQubitCircuit:
         qc.add_gate("T1", targets=[1])
         props = qc.propagators()
         result1 = tensor(identity(2), customer_gate1(np.pi/2))
-        np.testing.assert_allclose(props[0], result1)
+        np.testing.assert_allclose(props[0].full(), result1.full())
         result2 = tensor(identity(2), customer_gate2(), identity(2))
-        np.testing.assert_allclose(props[1], result2)
+        np.testing.assert_allclose(props[1].full(), result2.full())
 
     def test_N_level_system(self):
         """
         Test for circuit with N-level system.
         """
-        mat3 = rand_dm(3, density=1.)
+        mat3 = qp.rand_unitary(3)
 
         def controlled_mat3(arg_value):
             """
@@ -524,7 +561,13 @@ class TestQubitCircuit:
         qc.user_gates = {"CTRLMAT3": controlled_mat3}
         qc.add_gate("CTRLMAT3", targets=[1, 0], arg_value=1)
         props = qc.propagators()
-        np.testing.assert_allclose(mat3, ptrace(props[0], 0) - 1)
+        final_fid = qp.average_gate_fidelity(mat3, ptrace(props[0], 0) - 1)
+        assert pytest.approx(final_fid, 1.0e-6) == 1
+
+        init_state = basis([3, 2], [0, 1])
+        result = qc.run(init_state)
+        final_fid = qp.fidelity(result, props[0] * init_state)
+        assert pytest.approx(final_fid, 1.0e-6) == 1.
 
     @pytest.mark.repeat(10)
     def test_run_teleportation(self):
@@ -543,7 +586,6 @@ class TestQubitCircuit:
 
         teleportation_sim_results = teleportation_sim.run(state)
         state_final = teleportation_sim_results.get_final_states(0)
-        probability = teleportation_sim_results.get_probabilities(0)
 
         final_measurement = Measurement("start", targets=[2])
         _, final_probabilities = final_measurement.measurement_comp_basis(state_final)
@@ -600,6 +642,23 @@ class TestQubitCircuit:
                 else:
                     assert simulator.cbits[0] != simulator.cbits[1]
 
+    def test_circuit_with_selected_measurement_result(self):
+        qc = QubitCircuit(N=1, num_cbits=1)
+        qc.add_gate("SNOT", targets=0)
+        qc.add_measurement("M0", targets=0, classical_store=0)
+
+        # We reset the random seed so that
+        # if we don's select the measurement result,
+        # the two circuit should return the same value.
+        np.random.seed(0)
+        final_state = qc.run(qp.basis(2, 0), cbits=[0], measure_results=[0])
+        fid = pytest.approx(qp.fidelity(final_state, basis(2, 0)))
+        assert fid == 1.0
+        np.random.seed(0)
+        final_state = qc.run(qp.basis(2, 0), cbits=[0], measure_results=[1])
+        fid = pytest.approx(qp.fidelity(final_state, basis(2, 1)))
+        assert fid == 1.0
+
     def test_gate_product(self):
 
         filename = "qft.qasm"
@@ -615,7 +674,7 @@ class TestQubitCircuit:
             if isinstance(gate, Measurement):
                 continue
             else:
-                inds_list.append(gate.get_inds(qc.N))
+                inds_list.append(gate.get_all_qubits())
 
         U_1, _ = gate_sequence_product(U_list,
                                        inds_list=inds_list,
@@ -623,7 +682,7 @@ class TestQubitCircuit:
         U_2 = gate_sequence_product(U_list_expanded, left_to_right=True,
                                     expand=False)
 
-        np.testing.assert_allclose(U_1, U_2)
+        np.testing.assert_allclose(U_1.full(), U_2.full())
 
     def test_wstate(self):
 
@@ -632,10 +691,6 @@ class TestQubitCircuit:
         qc = read_qasm(filepath)
 
         rand_state = rand_ket(2)
-        wstate = (tensor(basis(2, 0), basis(2, 0), basis(2, 1))
-                  + tensor(basis(2, 0), basis(2, 1), basis(2, 0))
-                  + tensor(basis(2, 1), basis(2, 0), basis(2, 0))).unit()
-
         state = tensor(tensor(basis(2, 0), basis(2, 0), basis(2, 0)),
                        rand_state)
 
@@ -656,9 +711,8 @@ class TestQubitCircuit:
                 assert sum(result_cbits[i]) == 1
 
     _latex_template = r"""
-\documentclass{standalone}
+\documentclass[border=3pt]{standalone}
 \usepackage[braket]{qcircuit}
-\renewcommand{\qswap}{*=<0em>{\times}}
 \begin{document}
 \Qcircuit @C=1cm @R=1cm {
 %s}
@@ -681,3 +735,69 @@ class TestQubitCircuit:
             r"  \gate{{\rm H}}  &  \meter &  \qw  &  \qw  &  \qw  & \qw \\ ",
             "",
         ])
+
+    H = Qobj([[1/np.sqrt(2), 1/np.sqrt(2)], [1/np.sqrt(2), -1/np.sqrt(2)]])
+    H_zyz_gates = _ZYZ_rotation(H)
+    H_zyz_quantum_circuit = QubitCircuit(1)
+    H_zyz_quantum_circuit.add_gates(H_zyz_gates)
+    sigmax_zyz_gates = _ZYZ_rotation(sigmax())
+    sigmax_zyz_quantum_circuit = QubitCircuit(1)
+    sigmax_zyz_quantum_circuit.add_gates(sigmax_zyz_gates)
+
+    @pytest.mark.parametrize(
+        "valid_input, correct_result",
+        [(H_zyz_gates, H), 
+        (sigmax_zyz_gates, sigmax())]
+        )
+    def test_add_gates(self, valid_input, correct_result):
+        circuit = QubitCircuit(1)
+        circuit.add_gates(valid_input)
+        result = gate_sequence_product(circuit.propagators())
+        assert(result == correct_result)
+
+    @pytest.mark.parametrize(
+        "valid_input, correct_result",
+        [(H_zyz_quantum_circuit, H),
+        (sigmax_zyz_quantum_circuit, sigmax())]
+        )
+    def test_compute_unitary(
+            self, valid_input, correct_result):
+        final_output = valid_input.compute_unitary()
+        assert(isinstance(final_output, Qobj))
+        assert(final_output == correct_result)
+
+    def test_latex_code(self):
+        qc = QubitCircuit(1, num_cbits=1, reverse_states=True)
+        qc.add_measurement("M0", targets=0, classical_store=0)
+        exp = \
+            ' &  &  \\qw \\cwx[1]  & \\qw \\\\ \n &  &  \\meter & \\qw \\\\ \n'
+        assert qc.latex_code() == self._latex_template % exp
+
+    def test_latex_code_non_reversed(self):
+        qc = QubitCircuit(1, num_cbits=1, reverse_states=False)
+        qc.add_measurement("M0", targets=0, classical_store=0)
+        exp = ' &  &  \\meter & \\qw \\\\ \n &  ' + \
+              '&  \\qw \\cwx[-1]  & \\qw \\\\ \n'
+        assert qc.latex_code() == self._latex_template % exp
+
+    @pytest.mark.skipif(
+                shutil.which("pdflatex") is None,
+                reason="requires pdflatex"
+                )
+    def test_export_image(self, in_temporary_directory):
+        from qutip_qip import circuit_latex
+        qc = QubitCircuit(2, reverse_states=False)
+        qc.add_gate("CSIGN", controls=[0], targets=[1])
+
+        if "png" in circuit_latex.CONVERTERS:
+            file_png200 = "exported_pic_200.png"
+            file_png400 = "exported_pic_400.png"
+            qc.draw("png", 200, file_png200.split('.')[0], "")
+            qc.draw("png", 400.5, file_png400.split('.')[0], "")
+            assert file_png200 in os.listdir('.')
+            assert file_png400 in os.listdir('.')
+            assert os.stat(file_png200).st_size < os.stat(file_png400).st_size
+        if "svg" in circuit_latex.CONVERTERS:
+            file_svg = "exported_pic.svg"
+            qc.draw("svg", file_svg.split('.')[0], "")
+            assert file_svg in os.listdir('.')
