@@ -1,36 +1,5 @@
-# This file is part of QuTiP: Quantum Toolbox in Python.
-#
-#    Copyright (c) 2011 and later, Paul D. Nation and Robert J. Johansson.
-#    All rights reserved.
-#
-#    Redistribution and use in source and binary forms, with or without
-#    modification, are permitted provided that the following conditions are
-#    met:
-#
-#    1. Redistributions of source code must retain the above copyright notice,
-#       this list of conditions and the following disclaimer.
-#
-#    2. Redistributions in binary form must reproduce the above copyright
-#       notice, this list of conditions and the following disclaimer in the
-#       documentation and/or other materials provided with the distribution.
-#
-#    3. Neither the name of the QuTiP: Quantum Toolbox in Python nor the names
-#       of its contributors may be used to endorse or promote products derived
-#       from this software without specific prior written permission.
-#
-#    THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
-#    "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
-#    LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
-#    PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
-#    HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
-#    SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
-#    LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
-#    DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
-#    THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
-#    (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
-#    OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
-###############################################################################
-
+from copy import deepcopy
+from packaging.version import parse as parse_version
 import pytest
 import functools
 import itertools
@@ -48,23 +17,12 @@ def _infidelity(a, b):
     return 1 - abs(a.overlap(b))
 
 
-def _remove_global_phase(qobj):
-    """
-    Return a new Qobj with the gauge fixed for the global phase.  Explicitly,
-    we set the first non-zero element to be purely real-positive.
-    """
-    flat = qobj.full().flat.copy()
-    for phase in flat:
-        if phase != 0:
-            # Fix the gauge for any global phase.
-            flat = flat * np.exp(-1j * np.angle(phase))
-            break
-    return qutip.Qobj(flat.reshape(qobj.shape), dims=qobj.dims)
-
-
 def _make_random_three_qubit_gate():
     """Create a random three-qubit gate."""
-    operation = qutip.rand_unitary(8, dims=[[2]*3]*2)
+    if parse_version(qutip.__version__) < parse_version('5.dev'):
+        operation = qutip.rand_unitary(8, dims=[[2] * 3] * 2)
+    else:
+        operation = qutip.rand_unitary([2] * 3)
 
     def gate(N=None, controls=None, target=None):
         if N is None:
@@ -172,8 +130,8 @@ class TestExplicitForm:
         pytest.param(gates.qrot, 2, id="Rabi rotation"),
     ])
     def test_zero_rotations_are_identity(self, gate, n_angles):
-        np.testing.assert_allclose(np.eye(2), gate(*([0]*n_angles)),
-                                   atol=1e-15)
+        np.testing.assert_allclose(
+            np.eye(2), gate(*([0]*n_angles)).full(), atol=1e-15)
 
 
 class TestCliffordGroup:
@@ -188,11 +146,12 @@ class TestCliffordGroup:
         assert len(self.clifford) == 24
 
     def test_all_elements_different(self):
-        clifford = [_remove_global_phase(gate) for gate in self.clifford]
+        clifford = [gate for gate in self.clifford]
         for i, gate in enumerate(clifford):
             for other in clifford[i+1:]:
                 # Big tolerance because we actually want to test the inverse.
-                assert not np.allclose(gate.full(), other.full(), atol=1e-3)
+                fid = qutip.average_gate_fidelity(gate, other)
+                assert not np.allclose(fid, 1., atol=1e-3)
 
     @pytest.mark.parametrize("gate", gates.qubit_clifford_group())
     def test_gate_normalises_pauli_group(self, gate):
@@ -203,12 +162,12 @@ class TestCliffordGroup:
         # Assert that each Clifford gate maps the set of Pauli gates back onto
         # itself (though not necessarily in order).  This condition is no
         # stronger than simply considering each (gate, Pauli) pair separately.
-        pauli_gates = [_remove_global_phase(x) for x in self.pauli]
-        normalised = [_remove_global_phase(gate * pauli * gate.dag())
-                      for pauli in self.pauli]
+        pauli_gates = deepcopy(self.pauli)
+        normalised = [gate * pauli * gate.dag() for pauli in self.pauli]
         for gate in normalised:
             for i, pauli in enumerate(pauli_gates):
-                if np.allclose(gate.full(), pauli.full(), atol=1e-10):
+                # if np.allclose(gate.full(), pauli.full(), atol=1e-10):
+                if np.allclose(qutip.average_gate_fidelity(gate, pauli), 1):
                     del pauli_gates[i]
                     break
         assert len(pauli_gates) == 0
@@ -312,7 +271,10 @@ class Test_expand_operator:
     def test_general_qubit_expansion(self, n_targets):
         # Test all permutations with the given number of targets.
         n_qubits = 5
-        operation = qutip.rand_unitary(2**n_targets, dims=[[2]*n_targets]*2)
+        if parse_version(qutip.__version__) < parse_version('5.dev'):
+            operation = qutip.rand_unitary(2**n_targets, dims=[[2]*n_targets]*2)
+        else:
+            operation = qutip.rand_unitary([2]*n_targets)
         for targets in itertools.permutations(range(n_qubits), n_targets):
             expected = _tensor_with_entanglement([qutip.qeye(2)] * n_qubits,
                                                  operation, targets)
