@@ -1,95 +1,80 @@
-# This file is part of QuTiP: Quantum Toolbox in Python.
-#
-#    Copyright (c) 2011 and later, Paul D. Nation and Robert J. Johansson.
-#    All rights reserved.
-#
-#    Redistribution and use in source and binary forms, with or without
-#    modification, are permitted provided that the following conditions are
-#    met:
-#
-#    1. Redistributions of source code must retain the above copyright notice,
-#       this list of conditions and the following disclaimer.
-#
-#    2. Redistributions in binary form must reproduce the above copyright
-#       notice, this list of conditions and the following disclaimer in the
-#       documentation and/or other materials provided with the distribution.
-#
-#    3. Neither the name of the QuTiP: Quantum Toolbox in Python nor the names
-#       of its contributors may be used to endorse or promote products derived
-#       from this software without specific prior written permission.
-#
-#    THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
-#    "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
-#    LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
-#    PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
-#    HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
-#    SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
-#    LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
-#    DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
-#    THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
-#    (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
-#    OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
-###############################################################################
 import os
 
+from packaging.version import parse as parse_version
 from numpy.testing import (
     assert_, run_module_suite, assert_allclose, assert_equal)
 import numpy as np
+import pytest
 
-from qutip_qip.device.processor import Processor
-from qutip.states import basis
-from qutip.operators import sigmaz, sigmax, sigmay, identity, destroy
-from qutip_qip.operations.gates import hadamard_transform
-from qutip.tensor import tensor
-from qutip.solver import Options
-from qutip.random_objects import rand_ket, rand_dm
+import qutip
+if parse_version(qutip.__version__) < parse_version('5.dev'):
+    from qutip import Options as SolverOptions
+else:
+    from qutip import SolverOptions
+from qutip_qip.device import Processor, LinearSpinChain
+from qutip import (
+    basis, sigmaz, sigmax, sigmay, identity, destroy, tensor,
+    rand_ket, rand_dm, fidelity)
+from qutip_qip.operations import hadamard_transform
 from qutip_qip.noise import (
     DecoherenceNoise, RandomNoise, ControlAmpNoise)
 from qutip_qip.qubits import qubit_states
-from qutip.metrics import fidelity
 from qutip_qip.pulse import Pulse
+from qutip_qip.circuit import QubitCircuit
+if parse_version(qutip.__version__) < parse_version('5.dev'):
+    from qutip import Options as SolverOptions
+else:
+    from qutip import SolverOptions
 
 
 class TestCircuitProcessor:
-    def test_modify_ctrls(self):
-        """
-        Test for modifying Hamiltonian, add_control, remove_pulse
-        """
-        N = 2
-        proc = Processor(N=N)
-        proc.ctrls
-        proc.add_control(sigmaz())
-        assert_(tensor([sigmaz(), identity(2)]), proc.ctrls[0])
-        proc.add_control(sigmax(), cyclic_permutation=True)
-        assert_allclose(len(proc.ctrls), 3)
-        assert_allclose(tensor([sigmax(), identity(2)]), proc.ctrls[1])
-        assert_allclose(tensor([identity(2), sigmax()]), proc.ctrls[2])
-        proc.add_control(sigmay(), targets=1)
-        assert_allclose(tensor([identity(2), sigmay()]), proc.ctrls[3])
-        proc.remove_pulse([0, 1, 2])
-        assert_allclose(tensor([identity(2), sigmay()]), proc.ctrls[0])
-        proc.remove_pulse(0)
-        assert_allclose(len(proc.ctrls), 0)
+    def test_control_and_coeffs(self):
+        processor = Processor(2)
+        processor.add_control(sigmax())
+        processor.add_control(sigmaz())
+
+        # Set coeffs and tlist without a label
+        coeffs = np.array([[1., 2., 3.], [3., 2., 1.]])
+        processor.set_coeffs(coeffs)
+        assert_allclose(coeffs, processor.coeffs)
+        tlist = np.array([0., 1., 2., 3.])
+        processor.set_tlist(tlist)
+        assert_allclose(tlist, processor.get_full_tlist())
+        processor.set_tlist({0: tlist, 1: tlist})
+        assert_allclose(tlist, processor.get_full_tlist())
+
+        # Pulses
+        assert(len(processor.pulses) == 2)
+        assert(processor.find_pulse(0) == processor.pulses[0])
+        assert(processor.find_pulse(1) == processor.pulses[1])
+        with pytest.raises(KeyError):
+            processor.find_pulse("non_exist_pulse")
 
     def test_save_read(self):
         """
         Test for saving and reading a pulse matrix
         """
         proc = Processor(N=2)
-        proc.add_control(sigmaz(), cyclic_permutation=True)
+        proc.add_control(sigmaz(), label="sz")
+        proc.add_control(sigmax(), label="sx")
         proc1 = Processor(N=2)
-        proc1.add_control(sigmaz(), cyclic_permutation=True)
+        proc1.add_control(sigmaz(), label="sz")
+        proc1.add_control(sigmax(), label="sx")
         proc2 = Processor(N=2)
-        proc2.add_control(sigmaz(), cyclic_permutation=True)
+        proc2.add_control(sigmaz(), label="sz")
+        proc2.add_control(sigmax(), label="sx")
         # TODO generalize to different tlist
-        tlist = np.array([0., 0.1, 0.2, 0.3, 0.4, 0.5])
+        tlist = np.array([0.0, 0.1, 0.2, 0.3, 0.4, 0.5])
         amp1 = np.arange(0, 5, 1)
         amp2 = np.arange(5, 0, -1)
 
-        proc.pulses[0].tlist = tlist
-        proc.pulses[0].coeff = amp1
-        proc.pulses[1].tlist = tlist
-        proc.pulses[1].coeff = amp2
+        proc.set_all_coeffs(
+            {
+                label: amp
+                for label, amp in zip(proc.get_control_labels(), [amp1, amp2])
+            }
+        )
+        proc.set_all_tlist(tlist)
         proc.save_coeff("qutip_test_CircuitProcessor.txt")
         proc1.read_coeff("qutip_test_CircuitProcessor.txt")
         os.remove("qutip_test_CircuitProcessor.txt")
@@ -111,9 +96,10 @@ class TestCircuitProcessor:
         tlist = [0., 1., 2.]
         proc.add_pulse(Pulse(identity(2), 0, tlist, False))
         result = proc.run_state(
-            init_state, options=Options(store_final_state=True))
-        global_phase = init_state.data[0, 0]/result.final_state.data[0, 0]
-        assert_allclose(global_phase*result.final_state, init_state)
+            init_state, options=SolverOptions(store_final_state=True))
+        global_phase = init_state[0, 0]/result.final_state[0, 0]
+        assert_allclose(
+            global_phase*result.final_state.full(), init_state.full())
 
     def test_id_with_T1_T2(self):
         """
@@ -160,7 +146,7 @@ class TestCircuitProcessor:
             err_msg="Error in t1 & t2 simulation, "
                     "with t1={} and t2={}".format(t1, t2))
 
-    def testPlot(self):
+    def test_plot(self):
         """
         Test for plotting functions
         """
@@ -171,10 +157,10 @@ class TestCircuitProcessor:
         # step_func
         tlist = np.linspace(0., 2*np.pi, 20)
         processor = Processor(N=1, spline_kind="step_func")
-        processor.add_control(sigmaz())
-        processor.pulses[0].tlist = tlist
-        processor.pulses[0].coeff = np.array([np.sin(t) for t in tlist])
-        fig, _ = processor.plot_pulses()
+        processor.add_control(sigmaz(), label="sz")
+        processor.set_all_coeffs({"sz": np.array([np.sin(t) for t in tlist])})
+        processor.set_all_tlist(tlist)
+        fig, _ = processor.plot_pulses(use_control_latex=False)
         # testing under Xvfb with pytest-xvfb complains if figure windows are
         # left open, so we politely close it:
         plt.close(fig)
@@ -182,15 +168,20 @@ class TestCircuitProcessor:
         # cubic spline
         tlist = np.linspace(0., 2*np.pi, 20)
         processor = Processor(N=1, spline_kind="cubic")
-        processor.add_control(sigmaz())
-        processor.pulses[0].tlist = tlist
-        processor.pulses[0].coeff = np.array([np.sin(t) for t in tlist])
-        processor.plot_pulses()
-        fig, _ = processor.plot_pulses()
+        processor.add_control(sigmaz(), label="sz")
+        processor.set_all_coeffs({"sz": np.array([np.sin(t) for t in tlist])})
+        processor.set_all_tlist(tlist)
+        fig, _ = processor.plot_pulses(use_control_latex=False)
         # testing under Xvfb with pytest-xvfb complains if figure windows are
         # left open, so we politely close it:
         plt.close(fig)
 
+    @pytest.mark.skipif(
+        parse_version(qutip.__version__) >= parse_version('5.dev'),
+        reason=
+        "QobjEvo in qutip 5 changes significantly."
+        "A new test needs to be built separately."
+        )
     def testSpline(self):
         """
         Test if the spline kind is correctly transfered into
@@ -199,15 +190,14 @@ class TestCircuitProcessor:
         tlist = np.array([1, 2, 3, 4, 5, 6], dtype=float)
         coeff = np.array([1, 1, 1, 1, 1, 1], dtype=float)
         processor = Processor(N=1, spline_kind="step_func")
-        processor.add_control(sigmaz())
-        processor.pulses[0].tlist = tlist
-        processor.pulses[0].coeff = coeff
+        processor.add_control(sigmaz(), label="sz")
+        processor.set_all_coeffs({"sz": coeff})
+        processor.set_tlist(tlist)
 
         ideal_qobjevo, _ = processor.get_qobjevo(noisy=False)
         assert_(ideal_qobjevo.args["_step_func_coeff"])
         noisy_qobjevo, c_ops = processor.get_qobjevo(noisy=True)
         assert_(noisy_qobjevo.args["_step_func_coeff"])
-        processor.T1 = 100.
         processor.add_noise(ControlAmpNoise(coeff=coeff, tlist=tlist))
         noisy_qobjevo, c_ops = processor.get_qobjevo(noisy=True)
         assert_(noisy_qobjevo.args["_step_func_coeff"])
@@ -215,26 +205,32 @@ class TestCircuitProcessor:
         tlist = np.array([1, 2, 3, 4, 5, 6], dtype=float)
         coeff = np.array([1, 1, 1, 1, 1, 1], dtype=float)
         processor = Processor(N=1, spline_kind="cubic")
-        processor.add_control(sigmaz())
-        processor.pulses[0].tlist = tlist
-        processor.pulses[0].coeff = coeff
+        processor.add_control(sigmaz(), label="sz")
+        processor.set_all_coeffs({"sz": coeff})
+        processor.set_all_tlist(tlist)
 
         ideal_qobjevo, _ = processor.get_qobjevo(noisy=False)
-        assert_(not ideal_qobjevo.args["_step_func_coeff"])
+        assert ("_step_func_coeff" not in ideal_qobjevo.args)
         noisy_qobjevo, c_ops = processor.get_qobjevo(noisy=True)
-        assert_(not noisy_qobjevo.args["_step_func_coeff"])
-        processor.T1 = 100.
+        assert ("_step_func_coeff" not in noisy_qobjevo.args)
+        processor.t1 = 100.
         processor.add_noise(ControlAmpNoise(coeff=coeff, tlist=tlist))
         noisy_qobjevo, c_ops = processor.get_qobjevo(noisy=True)
-        assert_(not noisy_qobjevo.args["_step_func_coeff"])
+        assert ("_step_func_coeff" not in noisy_qobjevo.args)
 
+    @pytest.mark.skipif(
+        parse_version(qutip.__version__) >= parse_version('5.dev'),
+        reason=
+        "QobjEvo in qutip 5 changes significantly."
+        "A new test needs to be built separately."
+        )
     def testGetObjevo(self):
         tlist = np.array([1, 2, 3, 4, 5, 6], dtype=float)
         coeff = np.array([1, 1, 1, 1, 1, 1], dtype=float)
         processor = Processor(N=1)
-        processor.add_control(sigmaz())
-        processor.pulses[0].tlist = tlist
-        processor.pulses[0].coeff = coeff
+        processor.add_control(sigmaz(), label="sz")
+        processor.set_all_coeffs({"sz": coeff})
+        processor.set_all_tlist(tlist)
 
         # without noise
         unitary_qobjevo, _ = processor.get_qobjevo(
@@ -264,20 +260,18 @@ class TestCircuitProcessor:
 
         # with amplitude noise
         processor = Processor(N=1, spline_kind="cubic")
-        processor.add_control(sigmaz())
+        processor.add_control(sigmaz(), label="sz")
         tlist = np.linspace(1, 6, int(5/0.2))
         coeff = np.random.rand(len(tlist))
-        processor.pulses[0].tlist = tlist
-        processor.pulses[0].coeff = coeff
+        processor.set_all_coeffs({"sz": coeff})
+        processor.set_all_tlist(tlist)
 
         amp_noise = ControlAmpNoise(coeff=coeff, tlist=tlist)
         processor.add_noise(amp_noise)
         noisy_qobjevo, c_ops = processor.get_qobjevo(
             args={"test": True}, noisy=True)
-        assert_(not noisy_qobjevo.args["_step_func_coeff"],
-                msg="Spline type not correctly passed on")
-        assert_(noisy_qobjevo.args["test"],
-                msg="Arguments not correctly passed on")
+        assert ("_step_func_coeff" not in noisy_qobjevo.args)
+        assert noisy_qobjevo.args["test"]
         assert_equal(len(noisy_qobjevo.ops), 2)
         assert_equal(sigmaz(), noisy_qobjevo.ops[0].qobj)
         assert_allclose(coeff, noisy_qobjevo.ops[0].coeff, rtol=1.e-10)
@@ -291,24 +285,32 @@ class TestCircuitProcessor:
         tlist = np.array([0., np.pi/2.])
         a = destroy(2)
         proc = Processor(N=2)
-        proc.add_control(sigmax(), targets=1)
-        proc.pulses[0].tlist = tlist
-        proc.pulses[0].coeff = np.array([1])
+        proc.add_control(sigmax(), targets=1, label="sx")
+        proc.set_all_coeffs({"sx": np.array([1.])})
+        proc.set_all_tlist(tlist)
         result = proc.run_state(init_state=init_state)
         assert_allclose(
-            fidelity(result.states[-1], qubit_states(2, [0, 1, 0, 0])),
-            1, rtol=1.e-7)
+            fidelity(
+                result.states[-1],
+                qubit_states(2, [0, 1, 0, 0])
+            ),
+            1, rtol=1.e-7
+        )
 
         # decoherence noise
         dec_noise = DecoherenceNoise([0.25*a], targets=1)
         proc.add_noise(dec_noise)
         result = proc.run_state(init_state=init_state)
         assert_allclose(
-            fidelity(result.states[-1], qubit_states(2, [0, 1, 0, 0])),
-            0.981852, rtol=1.e-3)
+            fidelity(
+                result.states[-1],
+                qubit_states(2, [0, 1, 0, 0])
+            ),
+            0.981852, rtol=1.e-3
+        )
 
         # white random noise
-        proc.noise = []
+        proc.model._noise = []
         white_noise = RandomNoise(0.2, np.random.normal, loc=0.1, scale=0.1)
         proc.add_noise(white_noise)
         result = proc.run_state(init_state=init_state)
@@ -319,9 +321,9 @@ class TestCircuitProcessor:
         """
         N = 2
         proc = Processor(N=N, dims=[2, 3])
-        proc.add_control(tensor(sigmaz(), rand_dm(3, density=1.)))
-        proc.pulses[0].coeff = np.array([1, 2])
-        proc.pulses[0].tlist = np.array([0., 1., 2])
+        proc.add_control(tensor(sigmaz(), rand_dm(3, density=1.)), label="sz0")
+        proc.set_all_coeffs({"sz0": np.array([1, 2])})
+        proc.set_all_tlist(np.array([0., 1., 2]))
         proc.run_state(init_state=tensor([basis(2, 0), basis(3, 1)]))
 
     def testDrift(self):
@@ -333,7 +335,7 @@ class TestCircuitProcessor:
         tlist = np.array([0., np.pi, 2*np.pi, 3*np.pi])
         processor.add_pulse(Pulse(None, None, tlist, False))
         ideal_qobjevo, _ = processor.get_qobjevo(noisy=True)
-        assert_equal(ideal_qobjevo.cte, sigmax() / 2)
+        assert_equal(ideal_qobjevo(0), sigmax() / 2)
 
         init_state = basis(2)
         propagators = processor.run_analytically()
@@ -346,16 +348,16 @@ class TestCircuitProcessor:
     def testChooseSolver(self):
         # setup and fidelity without noise
         init_state = qubit_states(2, [0, 0, 0, 0])
-        tlist = np.array([0., np.pi/2.])
+        tlist = np.linspace(0., np.pi/2., 10)
         a = destroy(2)
-        proc = Processor(N=2)
-        proc.add_control(sigmax(), targets=1)
-        proc.pulses[0].tlist = tlist
-        proc.pulses[0].coeff = np.array([1])
-        result = proc.run_state(init_state=init_state, solver="mcsolve")
-        assert_allclose(
-            fidelity(result.states[-1], qubit_states(2, [0, 1, 0, 0])),
-            1, rtol=1.e-7)
+        proc = Processor(N=2, t2=100)
+        proc.add_control(sigmax(), targets=1, label="sx")
+        proc.set_all_coeffs({"sx": np.array([1.] * len(tlist))})
+        proc.set_all_tlist(tlist)
+        observerable = tensor([qutip.qeye(2), qutip.sigmax()])
+        result1 = proc.run_state(
+            init_state=init_state, solver="mcsolve", e_ops=observerable)
+        assert result1.solver == "mcsolve"
 
     def test_no_saving_intermidiate_state(self):
         processor = Processor(1)
@@ -366,11 +368,6 @@ class TestCircuitProcessor:
         result = processor.run_state(basis(2,0), tlist=[0,1])
         assert(len(result.states) == 2)
 
-    def test_pulse_dict(self):
-        processor = Processor(1)
-        processor.add_control(sigmax(), 0, label="test")
-        assert("test" in processor.get_pulse_dict())
-
     def test_repeated_use_of_processor(self):
         processor = Processor(1, t1=1.)
         processor.add_pulse(
@@ -378,3 +375,44 @@ class TestCircuitProcessor:
         result1 = processor.run_state(basis(2, 0), tlist=np.linspace(0, 1, 10))
         result2 = processor.run_state(basis(2, 0), tlist=np.linspace(0, 1, 10))
         assert_allclose(result1.states[-1].full(), result2.states[-1].full())
+
+    def test_pulse_mode(self):
+        processor = Processor(2)
+        processor.add_control(sigmax(), targets=0, label="sx")
+        processor.set_coeffs({"sx": np.array([1., 2., 3.])})
+        processor.set_tlist({"sx": np.array([0., 1., 2., 3.])})
+
+        processor.pulse_mode = "continuous"
+        assert(processor.pulse_mode == "continuous")
+        assert(processor.pulses[0].spline_kind == "cubic")
+        processor.pulse_mode = "discrete"
+        assert(processor.pulse_mode == "discrete")
+        assert(processor.pulses[0].spline_kind == "step_func")
+
+    def test_max_step_size(self):
+        num_qubits = 2
+        init_state = tensor([basis(2, 1), basis(2, 1)])
+        qc = QubitCircuit(2)
+
+        # ISWAP acts trivially on the initial states.
+        # If no max_step are defined,
+        # the solver will choose a step size too large
+        # such that the X gate will be skipped.
+        qc.add_gate("ISWAP", targets=[0, 1])
+        qc.add_gate("ISWAP", targets=[0, 1])
+        qc.add_gate("X", targets=[0])
+        processor = LinearSpinChain(num_qubits)
+        processor.load_circuit(qc)
+
+        # No max_step
+        final_state = processor.run_state(
+            init_state,
+            options=SolverOptions(max_step=10000)  # too large max_step
+        ).states[-1]
+        expected_state = tensor([basis(2, 0), basis(2, 1)])
+        assert pytest.approx(fidelity(final_state, expected_state), 0.001) == 0
+
+        # With default max_step
+        final_state = processor.run_state(init_state).states[-1]
+        expected_state = tensor([basis(2, 0), basis(2, 1)])
+        assert pytest.approx(fidelity(final_state, expected_state), 0.001) == 1
